@@ -143,7 +143,8 @@ dhcp6types = {   1:"SOLICIT",
 
 duidtypes = { 1: "Link-layer address plus time", 
               2: "Vendor-assigned unique ID based on Enterprise Number",
-              3: "Link-layer Address" }
+              3: "Link-layer Address",
+              4: "UUID" }
 
 # DUID hardware types - RFC 826 - Extracted from 
 # http://www.iana.org/assignments/arp-parameters on 31/10/06
@@ -232,9 +233,15 @@ class DUID_LL(Packet):  # sect 9.4 RFC 3315
                     XShortEnumField("hwtype", 1, duidhwtypes), 
                     _LLAddrField("lladdr", ETHER_ANY) ]
 
+class DUID_UUID(Packet):  # RFC 6355
+    name = "DUID - Based on UUID"
+    fields_desc = [ ShortEnumField("type", 4, duidtypes),
+                    StrFixedLenField("uuid","", 16) ]
+
 duid_cls = { 1: "DUID_LLT",
              2: "DUID_EN",
-             3: "DUID_LL"}
+             3: "DUID_LL",
+             4: "DUID_UUID"}
 
 #####################################################################
 ###                   DHCPv6 Options classes                      ###
@@ -256,7 +263,6 @@ class DHCP6OptUnknown(_DHCP6OptGuessPayload): # A generic DHCPv6 Option
                                 length_from = lambda pkt: pkt.optlen)]
 
 class _DUIDField(PacketField):
-    holds_packets=1
     def __init__(self, name, default, length_from=None):
         StrField.__init__(self, name, default)
         self.length_from = length_from
@@ -656,6 +662,11 @@ class DHCP6OptReconfAccept(_DHCP6OptGuessPayload):   # RFC sect 22.20
 #     Total length of domain should be 255 : we do not enforce it either
 class DomainNameListField(StrLenField):
     islist = 1
+    padded_unit = 8
+
+    def __init__(self, name, default, fld=None, length_from=None, padded=False):
+        self.padded = padded
+        StrLenField.__init__(self, name, default, fld, length_from)
 
     def i2len(self, pkt, x):
         return len(self.i2m(pkt, x))
@@ -663,12 +674,19 @@ class DomainNameListField(StrLenField):
     def m2i(self, pkt, x):
         res = []
         while x:
+            # Get a name until \x00 is reached
             cur = []
             while x and x[0] != '\x00':
                 l = ord(x[0])
                 cur.append(x[1:l+1])
                 x = x[l+1:]
-            res.append(".".join(cur))
+            if self.padded:
+              # Discard following \x00 in padded mode
+              if len(cur):
+                res.append(".".join(cur) + ".")
+            else:
+              # Store the current name
+              res.append(".".join(cur) + ".")
             if x and x[0] == '\x00':
                 x = x[1:]
         return res
@@ -678,9 +696,15 @@ class DomainNameListField(StrLenField):
             if z and z[-1] == '\x00':
                 return z
             return z+'\x00'
-        res = ""
+        # Build the encode names
         tmp = map(lambda y: map((lambda z: chr(len(z))+z), y.split('.')), x)
-        return "".join(map(lambda x: conditionalTrailingDot("".join(x)), tmp))
+        ret_string  = "".join(map(lambda x: conditionalTrailingDot("".join(x)), tmp))
+
+        # In padded mode, add some \x00 bytes
+        if self.padded and not len(ret_string) % self.padded_unit == 0:
+            ret_string += "\x00" * (self.padded_unit - len(ret_string) % self.padded_unit)
+
+        return ret_string
 
 class DHCP6OptSIPDomains(_DHCP6OptGuessPayload):       #RFC3319
     name = "DHCP6 Option - SIP Servers Domain Name List"
@@ -888,13 +912,6 @@ DHCP6PrefVal="" # la valeur de preference a utiliser dans
 #            INFORMATION REQUEST
 # - relay  : RELAY-FORW (toward server)
 
-class _DHCP6GuessPayload(Packet):
-    def guess_payload_class(self, payload):
-        if len(payload) > 1 :
-            print ord(payload[0])
-            return get_cls(dhcp6opts.get(ord(payload[0]),"DHCP6OptUnknown"), conf.raw_layer)
-        return conf.raw_layer
-
 #####################################################################
 ## DHCPv6 messages sent between Clients and Servers (types 1 to 11)
 # Comme specifie en section 15.1 de la RFC 3315, les valeurs de
@@ -1131,7 +1148,7 @@ class DHCP6_InfoRequest(DHCP6):
 # address or other multicast addresses, it sets the Hop Limit field to
 # 32. 
 
-class DHCP6_RelayForward(_DHCP6GuessPayload,Packet):
+class DHCP6_RelayForward(_DHCP6OptGuessPayload,Packet):
     name = "DHCPv6 Relay Forward Message (Relay Agent/Server Message)"
     fields_desc = [ ByteEnumField("msgtype", 12, dhcp6types),
                     ByteField("hopcount", None),
